<?php
/**
 * Copyright © 2015 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace Magento\Sales\Test\Unit\Helper;

use \Magento\Sales\Helper\Guest;

use Magento\Framework\TestFramework\Unit\Helper\ObjectManager as ObjectManagerHelper;

/**
 * Class GuestTest
 * @SuppressWarnings(PHPMD.CouplingBetweenObjects)
 */
class GuestTest extends \PHPUnit_Framework_TestCase
{
    /** @var \Magento\Sales\Helper\Guest */
    protected $guest;

    /** @var ObjectManagerHelper */
    protected $objectManagerHelper;

    /** @var \Magento\Framework\App\Helper\Context|\PHPUnit_Framework_MockObject_MockObject */
    protected $appContextHelperMock;

    /** @var \Magento\Store\Model\StoreManagerInterface|\PHPUnit_Framework_MockObject_MockObject */
    protected $storeManagerInterfaceMock;

    /** @var \Magento\Framework\Registry|\PHPUnit_Framework_MockObject_MockObject */
    protected $registryMock;

    /** @var \Magento\Customer\Model\Session|\PHPUnit_Framework_MockObject_MockObject */
    protected $sessionMock;

    /** @var \Magento\Framework\Stdlib\CookieManagerInterface|\PHPUnit_Framework_MockObject_MockObject */
    protected $cookieManagerMock;

    /** @var \Magento\Framework\Stdlib\Cookie\CookieMetadataFactory|\PHPUnit_Framework_MockObject_MockObject */
    protected $cookieMetadataFactoryMock;

    /** @var \Magento\Framework\Message\ManagerInterface|\PHPUnit_Framework_MockObject_MockObject */
    protected $managerInterfaceMock;

    /** @var \PHPUnit_Framework_MockObject_MockObject */
    protected $orderFactoryMock;

    /** @var \Magento\Framework\App\ViewInterface|\PHPUnit_Framework_MockObject_MockObject */
    protected $viewInterfaceMock;

    protected function setUp()
    {
        $this->appContextHelperMock = $this->getMock('Magento\Framework\App\Helper\Context', [], [], '', false);
        $this->storeManagerInterfaceMock = $this->getMock('Magento\Store\Model\StoreManagerInterface');
        $this->registryMock = $this->getMock('Magento\Framework\Registry');
        $this->sessionMock = $this->getMock('Magento\Customer\Model\Session', [], [], '', false);
        $this->cookieManagerMock = $this->getMock('Magento\Framework\Stdlib\CookieManagerInterface');
        $this->cookieMetadataFactoryMock = $this->getMock(
            'Magento\Framework\Stdlib\Cookie\CookieMetadataFactory',
            [],
            [],
            '',
            false
        );
        $this->managerInterfaceMock = $this->getMock('Magento\Framework\Message\ManagerInterface');
        $this->orderFactoryMock = $this->getMock('Magento\Sales\Model\OrderFactory', ['create'], [], '', false);
        $this->viewInterfaceMock = $this->getMock('Magento\Framework\App\ViewInterface');

        $this->objectManagerHelper = new ObjectManagerHelper($this);
        $this->guest = $this->objectManagerHelper->getObject(
            'Magento\Sales\Helper\Guest',
            [
                'context' => $this->appContextHelperMock,
                'storeManager' => $this->storeManagerInterfaceMock,
                'coreRegistry' => $this->registryMock,
                'customerSession' => $this->sessionMock,
                'cookieManager' => $this->cookieManagerMock,
                'cookieMetadataFactory' => $this->cookieMetadataFactoryMock,
                'messageManager' => $this->managerInterfaceMock,
                'orderFactory' => $this->orderFactoryMock,
                'view' => $this->viewInterfaceMock
            ]
        );
    }

    public function testLoadValidOrderNotEmptyPost()
    {
        $this->sessionMock->expects($this->once())->method('isLoggedIn')->will($this->returnValue(false));

        $post = [
            'oar_order_id' => 1,
            'oar_type' => 'email',
            'oar_billing_lastname' => 'oar_billing_lastname',
            'oar_email' => 'oar_email',
            'oar_zip' => 'oar_zip',

        ];
        $incrementId = $post['oar_order_id'];
        $requestMock = $this->getMock('Magento\Framework\App\Request\Http', [], [], '', false);
        $requestMock->expects($this->once())->method('getPostValue')->will($this->returnValue($post));

        $orderMock = $this->getMock(
            'Magento\Sales\Model\Order',
            ['getProtectCode', 'loadByIncrementId', 'getId', 'getBillingAddress', '__wakeup'],
            [],
            '',
            false
        );
        $this->orderFactoryMock->expects($this->once())->method('create')->will($this->returnValue($orderMock));
        $orderMock->expects($this->once())->method('loadByIncrementId')->with($incrementId);
        $orderMock->expects($this->exactly(2))->method('getId')->will($this->returnValue($incrementId));

        $billingAddressMock = $this->getMock(
            'Magento\Sales\Model\Order\Address',
            ['getLastname', 'getEmail', '__wakeup'],
            [],
            '',
            false
        );
        $billingAddressMock->expects($this->once())->method('getLastname')->will(
            $this->returnValue($post['oar_billing_lastname'])
        );
        $billingAddressMock->expects($this->once())->method('getEmail')->will(
            $this->returnValue($post['oar_email'])
        );
        $orderMock->expects($this->once())->method('getBillingAddress')->will($this->returnValue($billingAddressMock));
        $protectedCode = 'protectedCode';
        $orderMock->expects($this->once())->method('getProtectCode')->will($this->returnValue($protectedCode));
        $metaDataMock = $this->getMock(
            'Magento\Framework\Stdlib\Cookie\PublicCookieMetadata',
            [],
            [],
            '',
            false
        );
        $metaDataMock->expects($this->once())
            ->method('setPath')
            ->with(Guest::COOKIE_PATH)
            ->will($this->returnSelf());
        $metaDataMock->expects($this->once())
            ->method('setDuration')
            ->with(Guest::COOKIE_LIFETIME)
            ->will($this->returnSelf());
        $metaDataMock->expects($this->once())
            ->method('setHttpOnly')
            ->with(true)
            ->will($this->returnSelf());
        $this->cookieMetadataFactoryMock->expects($this->once())
            ->method('createPublicCookieMetadata')
            ->will($this->returnValue($metaDataMock));
        $this->cookieManagerMock->expects($this->once())
            ->method('setPublicCookie')
            ->with(Guest::COOKIE_NAME, $this->anything(), $metaDataMock);
        $this->assertTrue($this->guest->loadValidOrder($requestMock));
    }

    public function testLoadValidOrderStoredCookie()
    {
        $this->sessionMock->expects($this->once())->method('isLoggedIn')->will($this->returnValue(false));
        $orderMock = $this->getMock(
            'Magento\Sales\Model\Order',
            ['getProtectCode', 'loadByIncrementId', 'getId', 'getBillingAddress', '__wakeup'],
            [],
            '',
            false
        );
        $protectedCode = 'protectedCode';
        $incrementId = 1;
        $cookieData = $protectedCode . ':' . $incrementId;
        $cookieDataHash = base64_encode($cookieData);
        $this->orderFactoryMock->expects($this->once())->method('create')->will($this->returnValue($orderMock));

        $this->cookieManagerMock->expects($this->once())->method('getCookie')->with(Guest::COOKIE_NAME)->will(
            $this->returnValue($cookieDataHash)
        );
        $orderMock->expects($this->once())->method('loadByIncrementId')->with($incrementId);
        $orderMock->expects($this->exactly(1))->method('getId')->will($this->returnValue($incrementId));
        $orderMock->expects($this->once())->method('getProtectCode')->will($this->returnValue($protectedCode));
        $metaDataMock = $this->getMock(
            'Magento\Framework\Stdlib\Cookie\PublicCookieMetadata',
            [],
            [],
            '',
            false
        );
        $metaDataMock->expects($this->once())
            ->method('setPath')
            ->with(Guest::COOKIE_PATH)
            ->will($this->returnSelf());
        $metaDataMock->expects($this->once())
            ->method('setDuration')
            ->with(Guest::COOKIE_LIFETIME)
            ->will($this->returnSelf());
        $metaDataMock->expects($this->once())
            ->method('setHttpOnly')
            ->with(true)
            ->will($this->returnSelf());
        $this->cookieMetadataFactoryMock->expects($this->once())
            ->method('createPublicCookieMetadata')
            ->will($this->returnValue($metaDataMock));
        $this->cookieManagerMock->expects($this->once())
            ->method('setPublicCookie')
            ->with(Guest::COOKIE_NAME, $this->anything(), $metaDataMock);

        $requestMock = $this->getMock('Magento\Framework\App\Request\Http', [], [], '', false);
        $this->assertTrue($this->guest->loadValidOrder($requestMock));
    }
}
